!-------------------------------------------------------------------------------

!                      Code_Saturne version 2.0.1
!                      --------------------------

!     This file is part of the Code_Saturne Kernel, element of the
!     Code_Saturne CFD tool.

!     Copyright (C) 1998-2009 EDF S.A., France

!     contact: saturne-support@edf.fr

!     The Code_Saturne Kernel is free software; you can redistribute it
!     and/or modify it under the terms of the GNU General Public License
!     as published by the Free Software Foundation; either version 2 of
!     the License, or (at your option) any later version.

!     The Code_Saturne Kernel is distributed in the hope that it will be
!     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
!     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!     GNU General Public License for more details.

!     You should have received a copy of the GNU General Public License
!     along with the Code_Saturne Kernel; if not, write to the
!     Free Software Foundation, Inc.,
!     51 Franklin St, Fifth Floor,
!     Boston, MA  02110-1301  USA

!-------------------------------------------------------------------------------

subroutine usproj &
!================

 ( nvar   , nscal  ,                                              &
   nbpmax , nvp    , nvep   , nivep  , ntersl , nvlsta , nvisbr , &
   itepa  ,                                                       &
   dt     , rtpa   , rtp    , propce , propfa , propfb ,          &
   coefa  , coefb  ,                                              &
   ettp   , ettpa  , tepa   , statis , stativ , tslagr , parbor )

!===============================================================================
! Purpose:
! -------

!    User subroutine.

!    Called at end of each time step, very general purpose
!    (i.e. anything that does not have another dedicated user subroutine)


! Several examples are given here:

!  - compute a thermal balance
!    (if needed, see note  below on adapting this to any scalar)

!  - compute global efforts on a subset of faces

!  - arbitrarily modify a calculation variable

!  - extract a 1 d profile

!  - print a moment

!  - examples on using parallel utility functions

! These examples are valid when using periodicity (iperio .gt. 0)
! and in parallel (irangp .ge. 0).

! The thermal balance compution also illustates a few other features,
! including the required precautions in parallel or with periodicity):
! - gradient calculation
! - computation of a value depending on cells adjacent to a face
!   (see synchronization of Dt and Cp)
! - computation of a global sum in parallel (parsom)


! Cells, boundary faces and interior faces identification
! =======================================================

! Cells, boundary faces and interior faces may be identified using
! the subroutines 'getcel', 'getfbr' and 'getfac' (respectively).

!  getfbr(string, nelts, eltlst):
!  - string is a user-supplied character string containing selection criteria;
!  - nelts is set by the subroutine. It is an integer value corresponding to
!    the number of boundary faces verifying the selection criteria;
!  - lstelt is set by the subroutine. It is an integer array of size nelts
!    containing the list of boundary faces verifying the selection criteria.

!  string may contain:
!  - references to colors (ex.: 1, 8, 26, ...)
!  - references to groups (ex.: inlet, group1, ...)
!  - geometric criteria (ex. x < 0.1, y >= 0.25, ...)
!  These criteria may be combined using logical operators ('and', 'or') and
!  parentheses.
!  Example: '1 and (group2 or group3) and y < 1' will select boundary faces
!  of color 1, belonging to groups 'group2' or 'group3' and with face center
!  coordinate y less than 1.

! Similarly, interior faces and cells can be identified using the 'getfac'
! and 'getcel' subroutines (respectively). Their syntax are identical to
! 'getfbr' syntax.

! For a more thorough description of the criteria syntax, it can be referred
! to the user guide.


!-------------------------------------------------------------------------------
! Arguments
!__________________.____._____.________________________________________________.
! name             !type!mode ! role                                           !
!__________________!____!_____!________________________________________________!
! nvar             ! i  ! <-- ! total number of variables                      !
! nscal            ! i  ! <-- ! total number of scalars                        !
! nbpmax           ! i  ! <-- ! max. number of particles allowed               !
! nvp              ! i  ! <-- ! number of particle-defined variables           !
! nvep             ! i  ! <-- ! number of real particle properties             !
! nivep            ! i  ! <-- ! number of integer particle properties          !
! ntersl           ! i  ! <-- ! number of return coupling source terms         !
! nvlsta           ! i  ! <-- ! number of Lagrangian statistical variables     !
! nvisbr           ! i  ! <-- ! number of boundary statistics                  !
! itepa            ! ia ! <-- ! integer particle attributes                    !
!  (nbpmax, nivep) !    !     !   (containing cell, ...)                       !
! dt(ncelet)       ! ra ! <-- ! time step (per cell)                           !
! rtp, rtpa        ! ra ! <-- ! calculated variables at cell centers           !
!  (ncelet, *)     !    !     !  (at current and previous time steps)          !
! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
! propfa(nfac, *)  ! ra ! <-- ! physical properties at interior face centers   !
! propfb(nfabor, *)! ra ! <-- ! physical properties at boundary face centers   !
! coefa, coefb     ! ra ! <-- ! boundary conditions                            !
!  (nfabor, *)     !    !     !                                                !
! ettp, ettpa      ! ra ! <-- ! particle-defined variables                     !
!  (nbpmax, nvp)   !    !     !  (at current and previous time steps)          !
! tepa             ! ra ! <-- ! real particle properties                       !
!  (nbpmax, nvep)  !    !     !  (statistical weight, ...                      !
! statis           ! ra ! <-- ! statistic means                                !
!  (ncelet, nvlsta)!    !     !                                                !
! stativ(ncelet,   ! ra ! <-- ! accumulator for variance of volume statisitics !
!        nvlsta -1)!    !     !                                                !
! tslagr           ! ra ! <-- ! Lagrangian return coupling term                !
!  (ncelet, ntersl)!    !     !  on carrier phase                              !
! parbor           ! ra ! <-- ! particle interaction properties                !
!  (nfabor, nvisbr)!    !     !  on boundary faces                             !
!__________________!____!_____!________________________________________________!

!     Type: i (integer), r (real), s (string), a (array), l (logical),
!           and composite types (ex: ra real array)
!     mode: <-- input, --> output, <-> modifies data, --- work array
!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use dimens, only: ndimfb
use pointe
use numvar
use optcal
use cstphy
use cstnum
use entsor
use lagpar
use lagran
use parall
use period
use ppppar
use ppthch
use ppincl
use mesh

!===============================================================================

implicit none

! Arguments

integer          nvar   , nscal
integer          nbpmax , nvp    , nvep  , nivep
integer          ntersl , nvlsta , nvisbr

integer          itepa(nbpmax,nivep)

double precision dt(ncelet), rtp(ncelet,*), rtpa(ncelet,*)
double precision propce(ncelet,*)
double precision propfa(nfac,*), propfb(ndimfb,*)
double precision coefa(ndimfb,*), coefb(ndimfb,*)
double precision ettp(nbpmax,nvp) , ettpa(nbpmax,nvp)
double precision tepa(nbpmax,nvep)
double precision statis(ncelet,nvlsta), stativ(ncelet,nvlsta-1)
double precision tslagr(ncelet,ntersl)
double precision parbor(nfabor,nvisbr)


! Local variables

integer          iel    , ielg   , ifac   , ifacg  , ivar
integer          iel1   , iel2   , ieltsm
integer          iortho , impout
integer          inc    , iccocg
integer          nswrgp , imligp , iwarnp
integer          iutile , iclvar , iii
integer          ipcrom , ipcvst , iflmas , iflmab , ipccp, ipcvsl
integer          iscal
integer          ii     , nbr    , irangv , irang1 , npoint
integer          imom   , ipcmom , idtcm
integer          itab(3), iun
integer          ncesmp
integer          ilelt  , nlelt

double precision xrtpa  , xrtp
double precision xbilan , xbilvl , xbilpa , xbilpt
double precision xbilsy , xbilen , xbilso , xbildv
double precision xbilmi , xbilma
double precision epsrgp , climgp , extrap
double precision xfluxf , xgamma
double precision diipbx, diipby, diipbz, distbr
double precision visct, flumab , xcp , xvsl, rrr, ctb1, ctb2
double precision xfor(3), xyz(3), xabs, xu, xv, xw, xk, xeps

integer, allocatable, dimension(:) :: lstelt

double precision, allocatable, dimension(:,:) :: grad
double precision, allocatable, dimension(:) :: treco
DOUBLE PRECISION FX,FY,time,lastime,t_adim,dz,D,rho,V_ref, energy

!===============================================================================


!===============================================================================
! 1.  Initialization
!===============================================================================


! Allocate a temporary array for cells or interior/boundary faces selection
allocate(lstelt(max(ncel,nfac,nfabor)))

!===============================================================================
! 3. Example: compute global efforts on a subset of faces
!    ----------------------------------------------------
!===============================================================================

! ----------------------------------------------

! The test below allows checking that the following example compiles
! while disabling it by default.

      FX    = 0.D0
      FY    = 0.D0


      time = 0.D0
      lastime = 0.D0
      t_adim = 0.D0

!     valeurs fixees 
   
      DZ    = 1.0
      D     = 0.05
      rho   = 1000.0D0
      V_REF = 0.002D0
     
!c=======================================================================
!c     Calcul des efforts sur le cylindre
!c=======================================================================
!c     localisation du groupe representant le tube
!c     couleur 1 

!~ CALL GETFBR('wall',NLELT,LSTELT)
call getfbr("wall", nlelt,lstelt)
!C     ===========
!C      print *, IRANGP,NLELT,NRDEVE
DO ILELT = 1, NLELT
IFAC = LSTELT(ILELT)
FX = FX +forbr(1, ifac)
FY = FY + forbr(2, ifac)
ENDDO
!~ !c     
!~ !c     Si calcul parallele
!~ !c     
IF(IRANGP.GE.0) THEN
call PARSOM(FX)
call PARSOM(FY)
ENDIF

!c     
!~ t_adim = (TTCABS*V_REF)/D
!c     
!c======================================================================
!c     Ecriture des efforts dans un fichier
!c======================================================================
!c     -composition fichiers colonne1: NTCABS
!c     colonne2: TTCABS
!c     colonne3: valeur de la grandeur d'interet 
!c     
      
IF (IRANGP.LE.0) THEN
!c     
!c     IF (NTCABS.EQ.PDTDEB) THEN  ---> a rajouter si choix de dbut
!c     d ecriture different du debut du calcul
!c     
IF (ISUITE.EQ.0) THEN
IF (NTCABS.EQ.1) THEN 
OPEN(99, FILE = 'ForceX.hst', status='new')
OPEN(82, FILE = 'ForceY.hst', status='new')
ENDIF
ELSEIF(ISUITE.EQ.1) THEN
IF(NTCABS.EQ.NTPABS+1) THEN
OPEN(99, FILE = 'ForceX.hst', status='new',  &
POSITION='append')
OPEN(82, FILE = 'ForceY.hst', status='new',  &
POSITION='append')
ENDIF
ENDIF
!C     
!c--   Frequence d'ecriture dans le fichier = FREQECR
!c     IF (MOD(NTCABS,FREQECR).EQ.0) THEN
!c     
 WRITE (99,*) NTCABS, TTCABS,    &
FX
call flush(99)
WRITE (82,*) NTCABS, TTCABS, &
FY
call flush(82)
ENDIF
IF (NTCABS.EQ.NTMABS) THEN
WRITE(99,*) ' '
 WRITE(82,*) ' '
CLOSE(99)
CLOSE(82)
ENDIF

energy=0.0

      do iel = 1, ncel
energy=energy+volume(iel)*(RTP(IEL,IU)**2+   &
RTP(IEL,IV)**2+RTP(IEL,IW)**2)
       enddo
!C  - Somme des grandeurs sur tous les processeurs (calculs paralleles)
        IF (IRANGP.GE.0) THEN
        CALL PARSOM (energy)

        ENDIF
   
! 
!    -Ecriture a l instant N dans un fichier
        IF (IRANGP.LE.0) THEN
 OPEN(90, FILE = 'energy.hst', status='unknown',  &
 POSITION='append')
  WRITE (90,*) NTCABS, TTCABS,energy
call flush(90)
 	
IF (NTCABS.EQ.NTMABS) THEN
   WRITE(90,*) ' '
 
!     
   CLOSE(90)
 ENDIF
END IF
 
!c     
! 5110 FORMAT(i8,i8,10000(4x,E14.5))      
!c     
!c     
!c     
!c     
!c     
! 120  FORMAT(i8,197(3x,E14.8))      
!c     
! 130  FORMAT(i8,197(2x,E14.8))

! Deallocate the temporary array
deallocate(lstelt)
return
end subroutine
